/**
 * Copyright (c) 2017 Samsung Electronics Co., Ltd All Rights Reserved
 *
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
/**
 * @file
 * @author Lukasz Kostyra (l.kostyra@samsung.com)
 * @brief  TeecContext class methods definition
 */

#include "TeecContext.h"
#include "TeecTranslations.h"

#include <algorithm>
#include <common/platform_exception.h>


namespace {

// removes all dashes from UUID string to unify it (if there are any)
std::string UnifyUUID(const std::string& uuid) {
    std::string result = uuid;
    result.erase(std::remove(result.begin(), result.end(), '-'), result.end());
    return result;
}

// generates new ID for shared memory
std::string GenerateShmemID() {
    static uint64_t counter = 0;
    return "shmem-" + std::to_string(counter++);
}

} // namespace

namespace extension {
namespace libteec {

TeecContext::TeecContext(const std::string& name) {
    TEEC_Result ret = TEEC_InitializeContext(name.empty() ? NULL : name.c_str(), &mContext);
    if (ret != TEEC_SUCCESS) {
        throw common::NotSupportedException("Failed to initialize context: " +
                                            translations::TeecResultToString(ret));
    }
}

TeecContext::~TeecContext() {
    mSessions.clear();
    mSharedMemories.clear();
    TEEC_FinalizeContext(&mContext);
}

std::string TeecContext::OpenSession(const std::string& uuidstr, uint32_t connectionMethod,
                                     const void* connectionData, TEEC_Operation* operation,
                                     uint32_t* returnOrigin) {
    std::string uuid = UnifyUUID(uuidstr);
    auto it = mSessions.find(uuid);
    if (it == mSessions.end()) {
        mSessions.emplace(uuid, std::make_shared<TeecSession>(&mContext, uuid, connectionMethod,
                                                              connectionData, operation, returnOrigin));
        return uuid;
    } else {
        throw common::InvalidValuesException("Session for TA " + uuidstr + " already exists");
    }
}

TeecSessionPtr TeecContext::GetSession(const std::string& id) const {
    std::string uuid = UnifyUUID(id);
    auto it = mSessions.find(uuid);
    if (it == mSessions.end()) {
        throw common::InvalidValuesException("Session for TA " + id + " does not exist");
    }

    return it->second;
}

void TeecContext::CloseSession(const std::string& id) {
    std::string uuid = UnifyUUID(id);
    mSessions.erase(uuid);
}

std::string TeecContext::CreateSharedMemory(size_t size, uint32_t flags) {
    std::string id = GenerateShmemID();
    mSharedMemories.emplace(id, std::make_shared<TeecSharedMemory>(&mContext, size, flags));
    return id;
}

std::string TeecContext::CreateSharedMemory(void* buffer, size_t size, uint32_t flags) {
    std::string id = GenerateShmemID();
    mSharedMemories.emplace(id, std::make_shared<TeecSharedMemory>(&mContext, buffer, size, flags));
    return id;
}

TeecSharedMemoryPtr TeecContext::GetSharedMemory(const std::string& memId) const {
    auto it = mSharedMemories.find(memId);
    if (it == mSharedMemories.end()) {
        throw common::InvalidValuesException("Shared memory " + memId + " does not exist");
    }
    return it->second;
}

void TeecContext::ReleaseSharedMemory(const std::string& memId) {
    mSharedMemories.erase(memId);
}

} // namespace libteec
} // namespace extension
